from datetime import datetime
import glib
import os
import threading
import urllib
from keryx.unwrapt.utils import format_number


class InvalidCredentials(Exception):
    """
        Exception raised if the proxy credentials are invalid
    """
    
    pass

class ProxyOpener(urllib.FancyURLopener):
    """
        Class for handling proxy credentials
    """
    
    def __init__(self, proxy={}, usr=None, pwd=None):
        urllib.FancyURLopener.__init__(self, proxy)
        self.count = 0
        self.proxy = proxy
        self.usr = usr
        self.pwd = pwd
        
    def prompt_user_passwd(self, host, realm):
        """
            Override the FancyURLopener prompt and simply return what was given
            Raise an error if there is a problem
        """
        
        self.count += 1
        
        if self.count > 1:
            raise InvalidCredentials, "Unable to authenticate to proxy"
                    
        return (self.usr, self.pwd)


class DownloadThread(threading.Thread):

    def __init__(self, config, treeview):

	super(DownloadThread, self).__init__()
	print "Initializing download thread"

	self.config = config
	self.tv = treeview
	self.model = self.tv.get_model()

	self.running = True
	self.downloads = []


    def add(self, url, filename, callback):
        if self.running:
	        print "Added download: %s" % url
	        row = self.model.append((url, 0, "Waiting"))
	        self.downloads.append((url, filename, callback, row))

    
    def run(self):

        print "Starting download thread"

        while self.running and self.downloads:
            item = self.downloads[0]
            url, filename, callback, row = item
            print "Downloading %s" % url

            try:
                self.download(url, filename, row)
                glib.idle_add(callback, (url, filename, row), True)
                
            except Exception, e:
                print "Download error: %s" % e
		        #TODO: Error Dialog?

    	    self.downloads.remove(item)

        print "Downloads finished"
    
    def stop(self):

	print "Stopping download thread"
	self.running = False


    def download(self, url, filename, row):

	modified = None
	downloaded = 0
	filename = os.path.abspath(os.path.expanduser(filename))

	# Get data if we have a downloaded file already
	if os.path.exists(filename):
	    modified = datetime.utcfromtimestamp(os.stat(filename).st_mtime)
	    downloaded = os.path.getsize(filename)

	# Set our proxy every connection
	if self.config.get("proxy") == True:
	    proxy = {"http": "%s:%d" % (self.config.get("proxy_url"), int(float(self.config.get("proxy_port"))))}
	    username = self.config.get("proxy_username")
	    password = self.config.get("proxy_password")
	else:
	    proxy = {}
	    username = None
	    password = None

	print "Using proxy: %s" % proxy
	# Open a connection to see if we can use any downloaded data
	opener = ProxyOpener(proxy, username, password)
	headers = opener.open(url).headers

	if modified and "Last-Modified" in headers:
	    dt = datetime.strptime(headers["Last-Modified"],
	                           "%a, %d %b %Y %H:%M:%S %Z")

	    # File is too old so we delete the old file
	    if modified < dt:
		print "Deleting old copy of %s" % filename
		os.remove(filename)

	# Test existing filesize compared to total
	if downloaded and "Content-Length" in headers:
	    length = int(headers["Content-Length"])

	    if downloaded == length:
		self.set_progress(row, 100, 100)
		return

	    elif downloaded > length:
		downloaded = 0
		os.remove(filename)

	# Open up connection for downloading
	opener = ProxyOpener(proxy, username, password)
	if downloaded:
	    opener.addheader("Range", "bytes=%s-" % str(downloaded))
	page = opener.open(url)

	# The file range must match the downloaded size
	if not "Content-Length" in page.headers:
	    print "No Content-Length"
	    self.set_progress(row, 100, 100)
	    return

	# Finish downloading file
	length = int(page.headers["Content-Length"]) + downloaded
	f = open(filename, "ab")

	while self.running:
	    data = page.read(8192)

	    if not data:
		break

	    downloaded += len(data)
	    f.write(data)

	    self.set_progress(row, downloaded, length)
	
	if not self.running:
	    self.set_progress(row, downloaded, length, "Canceled")
	#else:
	#    self.set_progress(row, downloaded, length, "Finished")

	f.close()
	page.close()


    def set_progress(self, row, current, total, progress=None):
	percent = float(current)/float(total) * 100

	if not progress:
	    progress = "%5sB of %5sB" % (format_number(current),
	                                 format_number(total))
	
	self.model.set_value(row, 1, percent)
	self.model.set_value(row, 2, progress)
